'''
Copyright (c) 2012, 2013, 2014 The PyPedia Project, http://www.pypedia.com
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: 

# Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
# Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

http://www.opensource.org/licenses/BSD-2-Clause
'''

import argparse
import numpy as np

from time import gmtime, strftime

def print_now():
	return strftime("%a, %d %b %Y %H:%M:%S +0000", gmtime())

def imputation_allele_frequency(posteriori):
	"""
	>>> imputation_allele_frequency([[0, 0.9612, 0.0388], [0.7106, 0.2554, 0.034], [0, 0.008, 0.992], [0.9424, 0.057, 0.0006]])
	[0.57345, 0.42655]
	"""
	AF_A = 0.0
	AF_B = 0.0
	length = len(posteriori)
	for sample in posteriori:
			AF_A += (2.0 * sample[0]) + sample[1]
			AF_B += (2.0 * sample[2]) + sample[1]

	AF_A = AF_A / (2.0 * length)
	AF_B = AF_B / (2.0 * length)

	return [AF_A, AF_B]

def impute2_info(probs):
  """
  >>> p = [[0, 0.9612, 0.0388], [0.7106, 0.2554, 0.034], [0, 0.008, 0.992], [0.9424, 0.057, 0.0006]]
  >>> print impute2_info(p)
  0.801692726375
  """
  allele_freq = imputation_allele_frequency(probs)
  
  dosage = [x[1] + 2.0*x[2] for x in probs]
  dosage2 = [x[1] + 4.0*x[2] for x in probs]
  IA1 = sum([f-(e**2.0) for e,f in zip(dosage,dosage2)])
  den = 2.0 * float(len(probs)) * allele_freq[0] * allele_freq[1]
  if den < 0.00000001:
    return 1.0
  return 1.0 - (IA1 / den)


def convert_gprobs_info(input_gprobs_filename, output_info_filename):
	'''
	Generates .info from gprobs file (file that contains genotype probabilities) from impute2
	This task is be default also performed by impute2. This function can be useful in cases we 
	want to merge gprobs files. For example when we have split the imputation process in
	subprocesses where each one imputes a subset of the GWAS samples. 

	author: Alexandros Kanterakis, alexandros.kanterakis@gmail.com
	'''

	print 'Converting gpobs file %s to info' % input_gprobs_filename

	with open(input_gprobs_filename) as input_gprobs_file, open(output_info_filename, 'w') as output_info_file:

		header = "snp_id rs_id position exp_freq_a1 info certainty type"
		output_info_file.write(header + '\n')

		line_counter = 0
		for l in input_gprobs_file:
			line_counter += 1

			if line_counter % 10000 == 0:
				print 'Lines Converted:', line_counter, print_now()

			ls = l.replace('\n', '').split()

			id1 = ls[0]
			rscode = ls[1]
			pos = ls[2]

			values = [float(x) for x in ls[5:]]
			posteriori = [(values[i], values[i+1], values[i+2]) for i in range(0, len(values), 3)]

			exp_freq_a1 = imputation_allele_frequency(posteriori)[1]
			info = impute2_info(posteriori)
			certainty = np.mean([max(p) for p in posteriori])
			type_field = '0' if id1 == '---' else '2'

			to_print = [id1, rscode, pos, '%.3f' % exp_freq_a1, '%.3f' % info, '%.3f' % certainty, type_field]

			output_info_file.write(' '.join(to_print) + '\n')

	print 'Input gprobs filename:', input_gprobs_filename
	print 'Output info filename:', output_info_filename

if __name__ == '__main__':

	parser = argparse.ArgumentParser()

	parser.add_argument('--input_gprobs_filename', type=str, help='input gprobs filename')
	parser.add_argument('--output_info_filename', type=str, help='output info filename')

	args = parser.parse_args()

	convert_gprobs_info(args.input_gprobs_filename, args.output_info_filename)


